/**
 * OAuth specific functions.
 *
 * Part of the basebox sample Todo app.
 * https://basebox.tech
 */

import {store} from "../store";
import {gqlQuery} from "./net";

/**
 * Handle OAuth callback and complete login process.
 *
 * After the user enters her/his credentials at the IdP (Keycloak) login form, she/he
 * gets redirected to a client URL; this function handles this request.
 *
 * This function passes the query string received with the callback to the basebox broker,
 * which responds with a session information struct (JSON) like this:
 *
 * {
 *   "token":"47caa9df-ac89-45a8-8b22-9c0925d6aed0",
 *   "username":"tester",
 *   "first_name":"Fred",
 *   "last_name":"Feuerstein",
 *   "roles":[
 *     "/user"
 *   ],
 *   "claims":{
 *     "groups":[
 *       "/user"
 *     ]
 *   }
 * }
 *
 * The session is then stored in app state.
 *
 * Then, this function checks if the user is known in the database; since we use Keycloak/OpenID Connect
 * for user management, the user might be available in Keycloak, but not in the database. So if the user
 * is not there, it is created.
 *
 * @param queryString
 * @returns {Promise<void>}
 * @throws Error if the session data cannot be retrieved.
 */
export async function oauthCallbackHandler(queryString) {

  fetch(`${store.baseboxHost}/oauth/complete-login?${queryString}`, {
    method: "POST",
  }).then(response => {
    if (!response.ok) {
      throw new Error("Failed to get session data: " + response.statusText);
    }

    /* Store session data */
    response.json(
    ).then(rspJson => {
      store.session = rspJson;
      store.userName = store.session.first_name ? store.session.first_name : store.session.username;
    });

  }).catch(e => {
    throw new Error("Failed to get session data: " + e.toString());
  });
}

/**
 * Create the user on the broker/todo database.
 *
 * Since user management is done by Keycloak, which uses its own user database, we must
 * make sure that out user is known in the app database (basebox broker). After successful
 * login, this function must be called to do that.
 *
 * @param username unique username of new user
 * @param firstName first name of new user
 * @param lastName last name of new user
 * @returns {Promise<void>}
 */
async function createUser(username, firstName, lastName) {

  gqlQuery(`query {
    createUser(
      username: "${username}",
      name: "${firstName} ${lastName}"
    )`
  ).catch(err => {

  });


}